@@ -1,5 +1,6 @@
 #include <RcppEigen.h>
 #include "bvharmisc.h"
+#include "bvhardraw.h"
 
 //' Forecasting BVAR(p)
 //' 
@@ -174,6 +175,67 @@ Rcpp::List forecast_bvarssvs(int var_lag,
   );
 }
 
+//' Forecasting VAR(p) with Horseshoe Prior
+//' 
+//' @param var_lag VAR order.
+//' @param step Integer, Step to forecast.
+//' @param response_mat Response matrix.
+//' @param coef_mat Posterior mean of SSVS.
+//' @param alpha_record Matrix, MCMC trace of alpha.
+//' @param eta_record Matrix, MCMC trace of eta.
+//' @param omega_record Matrix, MCMC trace of omega.
+//' @noRd
+// [[Rcpp::export]]
+Rcpp::List forecast_bvarhs(int var_lag,
+                           int step,
+                           Eigen::MatrixXd response_mat,
+                           Eigen::MatrixXd coef_mat,
+                           Eigen::MatrixXd alpha_record,
+                           Eigen::MatrixXd eta_record,
+                           Eigen::MatrixXd omega_record) {
+  int num_sim = alpha_record.rows();
+  int dim = response_mat.cols();
+  int num_design = response_mat.rows();
+  int dim_design = coef_mat.rows();
+  Eigen::MatrixXd point_forecast(step, dim);
+  Eigen::VectorXd density_forecast(dim);
+  Eigen::MatrixXd predictive_distn(step, num_sim * dim);
+  Eigen::VectorXd last_pvec(dim_design);
+  Eigen::VectorXd tmp_vec((var_lag - 1) * dim);
+  last_pvec[dim_design - 1] = 1.0;
+  for (int i = 0; i < var_lag; i++) {
+    last_pvec.segment(i * dim, dim) = response_mat.row(num_design - 1 - i);
+  }
+  point_forecast.row(0) = last_pvec.transpose() * coef_mat;
+  Eigen::MatrixXd sig_cycle(dim, dim);
+  for (int b = 0; b < num_sim; b++) {
+    density_forecast = last_pvec.transpose() * unvectorize(alpha_record.row(b), dim_design, dim);
+    sig_cycle = build_cov(omega_record.row(b), eta_record.row(b));
+    predictive_distn.block(0, b * dim, 1, dim) = sim_mgaussian_chol(1, density_forecast, sig_cycle);
+  }
+  if (step == 1) {
+    return Rcpp::List::create(
+      Rcpp::Named("posterior_mean") = point_forecast,
+      Rcpp::Named("predictive") = predictive_distn
+    );
+  }
+  for (int i = 1; i < step; i++) {
+    tmp_vec = last_pvec.segment(0, (var_lag - 1) * dim);
+    last_pvec.segment(dim, (var_lag - 1) * dim) = tmp_vec;
+    last_pvec.segment(0, dim) = point_forecast.row(i - 1);
+    point_forecast.row(i) = last_pvec.transpose() * coef_mat;
+    for (int b = 0; b < num_sim; b++) {
+      density_forecast = last_pvec.transpose() * unvectorize(alpha_record.row(b), dim_design, dim);
+      sig_cycle = build_cov(omega_record.row(b), eta_record.row(b));
+      predictive_distn.block(i, b * dim, 1, dim) = sim_mgaussian_chol(1, density_forecast, sig_cycle);
+    }
+  }
+  return Rcpp::List::create(
+    Rcpp::Named("posterior_mean") = point_forecast,
+    Rcpp::Named("predictive") = predictive_distn
+  );
+}
+
 //' Forecasting VAR-SV
 //' 
 //' @param var_lag VAR order.
@@ -188,8 +250,6 @@ Eigen::MatrixXd forecast_bvarsv(int var_lag, int step, Eigen::MatrixXd response_
   int num_design = response_mat.rows();
   int dim_design = coef_mat.rows();
   Eigen::MatrixXd point_forecast(step, dim);
-  // Eigen::VectorXd density_forecast(dim);
-  // Eigen::MatrixXd predictive_distn(step, num_sim * dim);
   Eigen::VectorXd last_pvec(dim_design);
   Eigen::VectorXd tmp_vec((var_lag - 1) * dim);
   last_pvec[dim_design - 1] = 1.0;
@@ -208,3 +268,91 @@ Eigen::MatrixXd forecast_bvarsv(int var_lag, int step, Eigen::MatrixXd response_
   }
   return point_forecast;
 }
+
+//' Forecasting predictive density of VAR-SV
+//' 
+//' @param var_lag VAR order.
+//' @param step Integer, Step to forecast.
+//' @param response_mat Response matrix.
+//' @param coef_mat Posterior mean.
+//' @param alpha_record MCMC record of coefficients
+//' @param h_last_record MCMC record of log-volatilities in last time
+//' @param a_record MCMC record of contemporaneous coefficients
+//' @param sigh_record MCMC record of variance of log-volatilities
+//' 
+//' @noRd
+// [[Rcpp::export]]
+Rcpp::List forecast_bvarsv_density(int var_lag,
+                                   int step,
+                                   Eigen::MatrixXd response_mat,
+                                   Eigen::MatrixXd coef_mat,
+                                   Eigen::MatrixXd alpha_record,
+                                   Eigen::MatrixXd h_last_record,
+                                   Eigen::MatrixXd a_record,
+                                   Eigen::MatrixXd sigh_record) {
+  int num_sim = alpha_record.rows();
+  int dim = response_mat.cols();
+  int num_design = response_mat.rows();
+  int dim_design = coef_mat.rows();
+  Eigen::MatrixXd point_forecast(step, dim);
+  Eigen::VectorXd density_forecast(dim);
+  Eigen::MatrixXd predictive_distn(step, num_sim * dim);
+  Eigen::VectorXd last_pvec(dim_design);
+  Eigen::VectorXd tmp_vec((var_lag - 1) * dim);
+  Eigen::VectorXd sv_update(dim);
+  Eigen::MatrixXd sv_cov = Eigen::MatrixXd::Zero(dim, dim);
+  last_pvec[dim_design - 1] = 1.0;
+  for (int i = 0; i < var_lag; i++) {
+    last_pvec.segment(i * dim, dim) = response_mat.row(num_design - 1 - i);
+  }
+  point_forecast.row(0) = last_pvec.transpose() * coef_mat;
+  Eigen::MatrixXd contem_mat = Eigen::MatrixXd::Zero(dim, dim);
+  Eigen::MatrixXd tvp_lvol = Eigen::MatrixXd::Zero(dim, dim);
+  Eigen::MatrixXd tvp_prec(dim, dim);
+  for (int b = 0; b < num_sim; b++) {
+    density_forecast = last_pvec.transpose() * unvectorize(alpha_record.row(b), dim_design, dim);
+    sv_cov.diagonal() = 1 / sigh_record.row(b).array(); // covariance of h_t
+    sv_update = vectorize_eigen(
+      sim_mgaussian_chol(1, h_last_record.row(b), sv_cov)
+    ); // h_T+1 = h_T + u_T
+    tvp_lvol.diagonal() = 1 / sv_update.array().exp(); // Dt = diag(exp(h_t))
+    contem_mat = build_inv_lower(dim, a_record.row(b));
+    tvp_prec = contem_mat.transpose() * tvp_lvol * contem_mat; // L^T D_T  L
+    predictive_distn.block(0, b * dim, 1, dim) = sim_mgaussian_chol(
+      1,
+      density_forecast,
+      tvp_prec.inverse()
+    );
+  }
+  if (step == 1) {
+    return Rcpp::List::create(
+      Rcpp::Named("posterior_mean") = point_forecast,
+      Rcpp::Named("predictive") = predictive_distn
+    );
+  }
+  for (int i = 1; i < step; i++) {
+    tmp_vec = last_pvec.segment(0, (var_lag - 1) * dim);
+    last_pvec.segment(dim, (var_lag - 1) * dim) = tmp_vec;
+    last_pvec.segment(0, dim) = point_forecast.row(i - 1);
+    point_forecast.row(i) = last_pvec.transpose() * coef_mat;
+    for (int b = 0; b < num_sim; b++) {
+      density_forecast = last_pvec.transpose() * unvectorize(alpha_record.row(b), dim_design, dim);
+      sv_cov.diagonal() = 1 / sigh_record.row(b).array();
+      sv_update = vectorize_eigen(
+        sim_mgaussian_chol(1, h_last_record.row(b), sv_cov)
+      );
+      tvp_lvol.diagonal() = 1 / sv_update.array();
+      contem_mat = build_inv_lower(dim, a_record.row(b));
+      tvp_prec = contem_mat.transpose() * tvp_lvol * contem_mat;
+      predictive_distn.block(i, b * dim, 1, dim) = sim_mgaussian_chol(
+        1,
+        density_forecast,
+        tvp_prec.inverse()
+      );
+    }
+  }
+  return Rcpp::List::create(
+    Rcpp::Named("posterior_mean") = point_forecast,
+    Rcpp::Named("predictive") = predictive_distn
+  );
+}
