@@ -1,5 +1,6 @@
 #include <RcppEigen.h>
 #include "bvharmisc.h"
+#include "bvhardraw.h"
 
 //' Forecasting Bayesian VHAR
 //' 
@@ -128,13 +129,14 @@ Rcpp::List forecast_bvharssvs(int month,
   int num_sim = phi_record.rows();
   int dim = response_mat.cols();
   int num_design = response_mat.rows();
-  int dim_har = HARtrans.cols();
+  int lag_var = HARtrans.cols();
+  int dim_har = HARtrans.rows();
   Eigen::MatrixXd point_forecast(step, dim);
   Eigen::VectorXd density_forecast(dim);
   Eigen::MatrixXd predictive_distn(step, num_sim * dim);
-  Eigen::VectorXd last_pvec(dim_har);
+  Eigen::VectorXd last_pvec(lag_var);
   Eigen::VectorXd tmp_vec((month - 1) * dim);
-  last_pvec[dim_har - 1] = 1.0;
+  last_pvec[lag_var - 1] = 1.0;
   for (int i = 0; i < month; i++) {
     last_pvec.segment(i * dim, dim) = response_mat.row(num_design - 1 - i);
   }
@@ -171,6 +173,70 @@ Rcpp::List forecast_bvharssvs(int month,
   );
 }
 
+//' Forecasting VHAR with Horseshoe Prior
+//' 
+//' @param month VHAR month order.
+//' @param step Integer, Step to forecast.
+//' @param response_mat Response matrix.
+//' @param coef_mat Posterior mean of SSVS.
+//' @param HARtrans VHAR linear transformation matrix
+//' @param phi_record Matrix, MCMC trace of phi.
+//' @param eta_record Matrix, MCMC trace of eta.
+//' @param omega_record Matrix, MCMC trace of omega.
+//' @noRd
+// [[Rcpp::export]]
+Rcpp::List forecast_bvharhs(int month,
+                            int step,
+                            Eigen::MatrixXd response_mat,
+                            Eigen::MatrixXd coef_mat,
+                            Eigen::MatrixXd HARtrans,
+                            Eigen::MatrixXd phi_record,
+                            Eigen::MatrixXd eta_record,
+                            Eigen::MatrixXd omega_record) {
+  int num_sim = phi_record.rows();
+  int dim = response_mat.cols();
+  int num_design = response_mat.rows();
+  int lag_var = HARtrans.cols();
+  int dim_har = HARtrans.rows();
+  Eigen::MatrixXd point_forecast(step, dim);
+  Eigen::VectorXd density_forecast(dim);
+  Eigen::MatrixXd predictive_distn(step, num_sim * dim);
+  Eigen::VectorXd last_pvec(lag_var);
+  Eigen::VectorXd tmp_vec((month - 1) * dim);
+  last_pvec[lag_var - 1] = 1.0;
+  for (int i = 0; i < month; i++) {
+    last_pvec.segment(i * dim, dim) = response_mat.row(num_design - 1 - i);
+  }
+  point_forecast.row(0) = last_pvec.transpose() * HARtrans.transpose() * coef_mat;
+  Eigen::MatrixXd sig_cycle(dim, dim);
+  for (int b = 0; b < num_sim; b++) {
+    density_forecast = last_pvec.transpose() * HARtrans.transpose() * unvectorize(phi_record.row(b), dim_har, dim);
+    sig_cycle = build_cov(omega_record.row(b), eta_record.row(b));
+    predictive_distn.block(0, b * dim, 1, dim) = sim_mgaussian_chol(1, density_forecast, sig_cycle);
+  }
+  if (step == 1) {
+    return Rcpp::List::create(
+      Rcpp::Named("posterior_mean") = point_forecast,
+      Rcpp::Named("predictive") = predictive_distn
+    );
+  }
+  for (int i = 1; i < step; i++) {
+    tmp_vec = last_pvec.segment(0, (month - 1) * dim);
+    last_pvec.segment(dim, (month - 1) * dim) = tmp_vec;
+    last_pvec.segment(0, dim) = point_forecast.row(i - 1);
+    point_forecast.row(i) = last_pvec.transpose() * HARtrans.transpose() * coef_mat;
+    for (int b = 0; b < num_sim; b++) {
+      density_forecast = last_pvec.transpose() * HARtrans.transpose() * unvectorize(phi_record.row(b), dim_har, dim);
+      sig_cycle = build_cov(omega_record.row(b), eta_record.row(b));
+      predictive_distn.block(i, b * dim, 1, dim) = sim_mgaussian_chol(1, density_forecast, sig_cycle);
+    }
+  }
+  return Rcpp::List::create(
+    Rcpp::Named("posterior_mean") = point_forecast,
+    Rcpp::Named("predictive") = predictive_distn
+  );
+}
+
 //' Forecasting VHAR-SV
 //' 
 //' @param month VHAR month order.
@@ -186,8 +252,6 @@ Eigen::MatrixXd forecast_bvharsv(int month, int step, Eigen::MatrixXd response_m
   int num_design = response_mat.rows();
   int dim_har = HARtrans.cols();
   Eigen::MatrixXd point_forecast(step, dim);
-  // Eigen::VectorXd density_forecast(dim);
-  // Eigen::MatrixXd predictive_distn(step, num_sim * dim);
   Eigen::VectorXd last_pvec(dim_har);
   Eigen::VectorXd tmp_vec((month - 1) * dim);
   last_pvec[dim_har - 1] = 1.0;
@@ -206,3 +270,90 @@ Eigen::MatrixXd forecast_bvharsv(int month, int step, Eigen::MatrixXd response_m
   }
   return point_forecast;
 }
+
+//' Forecasting Predictive Density of VHAR-SV
+//' 
+//' @param month VHAR month order.
+//' @param step Integer, Step to forecast.
+//' @param response_mat Response matrix.
+//' @param coef_mat Posterior mean.
+//' @param HARtrans VHAR linear transformation matrix
+//' 
+//' @noRd
+// [[Rcpp::export]]
+Rcpp::List forecast_bvharsv_density(int month,
+                                    int step,
+                                    Eigen::MatrixXd response_mat,
+                                    Eigen::MatrixXd coef_mat,
+                                    Eigen::MatrixXd HARtrans,
+                                    Eigen::MatrixXd phi_record,
+                                    Eigen::MatrixXd h_last_record,
+                                    Eigen::MatrixXd a_record,
+                                    Eigen::MatrixXd sigh_record) {
+  int num_sim = phi_record.rows();
+  int dim = response_mat.cols();
+  int num_design = response_mat.rows();
+  int lag_var = HARtrans.cols();
+  int dim_har = HARtrans.rows();
+  Eigen::MatrixXd point_forecast(step, dim);
+  Eigen::VectorXd density_forecast(dim);
+  Eigen::MatrixXd predictive_distn(step, num_sim * dim);
+  Eigen::VectorXd last_pvec(lag_var);
+  Eigen::VectorXd tmp_vec((month - 1) * dim);
+  Eigen::VectorXd sv_update(dim);
+  Eigen::MatrixXd sv_cov = Eigen::MatrixXd::Zero(dim, dim);
+  last_pvec[lag_var - 1] = 1.0;
+  for (int i = 0; i < month; i++) {
+    last_pvec.segment(i * dim, dim) = response_mat.row(num_design - 1 - i);
+  }
+  point_forecast.row(0) = last_pvec.transpose() * HARtrans.transpose() * coef_mat;
+  Eigen::MatrixXd contem_mat = Eigen::MatrixXd::Zero(dim, dim);
+  Eigen::MatrixXd tvp_lvol = Eigen::MatrixXd::Zero(dim, dim);
+  Eigen::MatrixXd tvp_prec(dim, dim);
+  for (int b = 0; b < num_sim; b++) {
+    density_forecast = last_pvec.transpose() * HARtrans.transpose() * unvectorize(phi_record.row(b), dim_har, dim);
+    sv_cov.diagonal() = 1 / sigh_record.row(b).array(); // covariance of h_t
+    sv_update = vectorize_eigen(
+      sim_mgaussian_chol(1, h_last_record.row(b), sv_cov)
+    ); // h_T+1 = h_T + u_T
+    tvp_lvol.diagonal() = 1 / sv_update.array();
+    contem_mat = build_inv_lower(dim, a_record.row(b));
+    tvp_prec = contem_mat.transpose() * tvp_lvol * contem_mat; // L^T D_T  L
+    predictive_distn.block(0, b * dim, 1, dim) = sim_mgaussian_chol(
+      1,
+      density_forecast,
+      tvp_prec.inverse()
+    );
+  }
+  if (step == 1) {
+    return Rcpp::List::create(
+      Rcpp::Named("posterior_mean") = point_forecast,
+      Rcpp::Named("predictive") = predictive_distn
+    );
+  }
+  for (int i = 1; i < step; i++) {
+    tmp_vec = last_pvec.segment(0, (month - 1) * dim);
+    last_pvec.segment(dim, (month - 1) * dim) = tmp_vec;
+    last_pvec.segment(0, dim) = point_forecast.row(i - 1);
+    point_forecast.row(i) = last_pvec.transpose() * HARtrans.transpose() * coef_mat;
+    for (int b = 0; b < num_sim; b++) {
+      density_forecast = last_pvec.transpose() * HARtrans.transpose() * unvectorize(phi_record.row(b), dim_har, dim);
+      sv_cov.diagonal() = 1 / sigh_record.row(b).array(); // covariance of h_t
+      sv_update = vectorize_eigen(
+        sim_mgaussian_chol(1, h_last_record.row(b), sv_cov)
+      ); // h_T+1 = h_T + u_T
+      tvp_lvol.diagonal() = 1 / sv_update.array().exp();
+      contem_mat = build_inv_lower(dim, a_record.row(b));
+      tvp_prec = contem_mat.transpose() * tvp_lvol * contem_mat; // L^T D_T  L
+      predictive_distn.block(i, b * dim, 1, dim) = sim_mgaussian_chol(
+        1,
+        density_forecast,
+        tvp_prec.inverse()
+      );
+    }
+  }
+  return Rcpp::List::create(
+    Rcpp::Named("posterior_mean") = point_forecast,
+    Rcpp::Named("predictive") = predictive_distn
+  );
+}
