@@ -1,4 +1,5 @@
 #include <RcppEigen.h>
+#include "bvhardraw.h"
 
 //' Numerically Stable Log Marginal Likelihood Excluding Constant Term
 //' 
@@ -163,3 +164,70 @@ Eigen::MatrixXd tune_var(Eigen::MatrixXd y, int lag_max, bool include_mean) {
   return ic_res;
 }
 
+//' log Density of Multivariate Normal with LDLT Precision Matrix
+//' 
+//' Compute log density of multivariate normal with LDLT precision matrix decomposition.
+//' 
+//' @param x Point
+//' @param mean_vec Mean
+//' @param lower_vec row of a_record
+//' @param diag_vec row of h_record
+//' 
+//' @noRd
+// [[Rcpp::export]]
+double compute_log_dmgaussian(Eigen::VectorXd x,
+                              Eigen::VectorXd mean_vec,
+                              Eigen::VectorXd lower_vec,
+                              Eigen::VectorXd diag_vec) {
+  int dim = diag_vec.size();
+  Eigen::MatrixXd diag_mat = Eigen::MatrixXd::Zero(dim, dim); // sqrt(D) in LDLT
+  diag_mat.diagonal() = 1 / diag_vec.array().exp().sqrt(); // exp since D = exp(h)
+  Eigen::MatrixXd lower_mat = build_inv_lower(dim, lower_vec);
+  x.array() -= mean_vec.array(); // x - mu
+  Eigen::VectorXd y = diag_mat * lower_mat * x; // sqrt(D) * L * (x - mu)
+  double res = -log(lower_vec.squaredNorm()) - log(diag_vec.sum()) / 2 - dim * log(2 * M_PI) / 2; // should fix this line?
+  res -= y.squaredNorm() / 2;
+  return res;
+}
+
+//' Compute Log Predictive Likelihood
+//' 
+//' This function computes log-predictive likelihood (LPL).
+//' 
+//' @param True value
+//' @param Predicted value
+//' @param h_last_record MCMC record of log-volatilities in last time
+//' @param a_record MCMC record of contemporaneous coefficients
+//' @param sigh_record MCMC record of variance of log-volatilities
+//' 
+//' @noRd
+// [[Rcpp::export]]
+Eigen::VectorXd compute_lpl(Eigen::MatrixXd y,
+                            Eigen::MatrixXd posterior_mean,
+                            Eigen::MatrixXd h_last_record,
+                            Eigen::MatrixXd a_record,
+                            Eigen::MatrixXd sigh_record) {
+  int num_sim = a_record.rows();
+  int dim = h_last_record.cols();
+  int num_pred = y.rows();
+  Eigen::VectorXd lpl_hist = Eigen::VectorXd::Zero(num_pred);
+  Eigen::VectorXd sv_update(dim);
+  Eigen::MatrixXd sv_cov = Eigen::MatrixXd::Zero(dim, dim);
+  for (int i = 0; i < num_pred; i++) {
+    for (int b = 0; b < num_sim; b++) {
+      sv_cov.diagonal() = 1 / sigh_record.row(b).array();
+      sv_update = vectorize_eigen(
+        sim_mgaussian_chol(1, h_last_record.row(b), sv_cov)
+      );
+      lpl_hist[i] += compute_log_dmgaussian(
+        y.row(i),
+        posterior_mean.row(i),
+        a_record.row(b),
+        sv_update
+      );
+    }
+    lpl_hist[i] /= num_sim;
+  }
+  // return lpl_hist.mean();
+  return lpl_hist;
+}
